#!/usr/bin/perl
use warnings;
use strict;

my $index = shift;
$index = 0
  unless (defined $index);

die("Usage: $0 [numeric-index]\n")
  unless ($index =~ /^\d+$/);

my $fname = sprintf("/dev/xillyusb_%02d_diagnostics", $index);

local $/; # Slurp

print "\033[2J"; # ANSI escape code for "Clear screen"

my @twiddle = qw[ | / - \ ];
my $twidpos = 0;

my @firstwords;

eval {
  local $SIG{ALRM} = sub { die "Timed out on receiving data.\n" };

  while (1) {
    alarm 3;

    open (my $fh, "<", $fname) or
      die("Failed to open $fname for read: $!\n");
    my $data = <$fh>;
    close $fh or
      die("Error while reading from $fname: $!\n");

    my @words = unpack("V*", $data);
    my $version = $words[0];

    die("Diagnostic data too short\n")
      if (@words < 8);

    die("Unexpected diagnostics data version $version, should be 1\n")
      unless ($version == 1);

    my $power_ctrl = $words[8];
    my $u2_timeout = $power_ctrl & 0xff;
    my $u2_timeout_text = ($u2_timeout * 0.256)." ms";
    $u2_timeout_text = '(disabled)'
      if (($u2_timeout == 0) || ($u2_timeout == 0xff));

    my $snooziness = 'Balanced (agree to low power state when no device file is open)';

    $snooziness = 'REFUSE to low power state'
      if ($power_ctrl & 0x10000);

    $snooziness = 'AGREE to low power state'
      if ($power_ctrl & 0x20000);

    @firstwords = @words unless (@firstwords);

    for (my $i=0; $i<8; $i++) {
      $words[$i] -= $firstwords[$i];
      $words[$i] &= 0xffffffff;
    }

    my $date = localtime;

    my $output = << "OUTPUT";
XillyUSB event counts $twiddle[$twidpos] $date

Errors on FPGA to host link:
===========================
Bad packets received by link partner: $words[2]
Recovery requests by link partner: $words[1]

Errors on host to FPGA link:
===========================
Bad packet received by device: $words[4]
Errors detected during link idle: $words[5]
Recovery requests by device: $words[3]

Power management:
================
XillyUSB device's power policy:
$snooziness

PORT_U2_TIMEOUT: $u2_timeout_text

Low power transitions made: $words[6]
Low power transitions refused by device: $words[7]
OUTPUT

    # Clear anything to the right of cursor, before all newlines
    $output =~ s/\n/"\033[0K\n"/ge;

    print "\033[1;1H"; # ANSI escape code for move cursor to home
    print $output;

    $twidpos = ($twidpos + 1) & 3;

    # Sleep 250 ms. sleep() doesn't support less than one second.
    select(undef, undef, undef, 0.25);
  }
};

if ($@) {
  print "\033[1000;1H"; # Go to bottom of screen
  print "$@";
  exit(1);
}
